from django.db import models
from django.core.exceptions import ValidationError
from os import listdir


class Location(models.Model):
    path = models.CharField(max_length=300)

    # for Public
    active_p = models.BooleanField(default=True, verbose_name="Active")
    show_f_p = models.BooleanField(default=True, verbose_name="Show Files")
    show_d_p = models.BooleanField(default=True, verbose_name="Show Directories")
    show_h_f_p = models.BooleanField(default=False, verbose_name="Show Hidden Files")
    show_h_d_p = models.BooleanField(default=False, verbose_name="Show Hidden Directories")
    can_down_p = models.BooleanField(default=False, verbose_name="Can Download")
    can_up_p = models.BooleanField(default=False, verbose_name="Can Upload")
    can_del_p = models.BooleanField(default=False, verbose_name="Can Delete")
    can_crt_p = models.BooleanField(default=False, verbose_name="Can Create")
    can_run_p = models.BooleanField(default=False, verbose_name="Can Run Command")

    # for Users
    active_u = models.BooleanField(default=True, verbose_name="Active")
    show_f_u = models.BooleanField(default=True, verbose_name="Show Files")
    show_d_u = models.BooleanField(default=True, verbose_name="Show Directories")
    show_h_f_u = models.BooleanField(default=False, verbose_name="Show Hidden Files")
    show_h_d_u = models.BooleanField(default=False, verbose_name="Show Hidden Directories")
    can_down_u = models.BooleanField(default=True, verbose_name="Can Download")
    can_up_u = models.BooleanField(default=True, verbose_name="Can Upload")
    can_del_u = models.BooleanField(default=False, verbose_name="Can Delete")
    can_crt_u = models.BooleanField(default=True, verbose_name="Can Create")
    can_run_u = models.BooleanField(default=False, verbose_name="Can Run Command")

    # for Admins
    active_a = models.BooleanField(default=True, verbose_name="Active")
    show_f_a = models.BooleanField(default=True, verbose_name="Show Files")
    show_d_a = models.BooleanField(default=True, verbose_name="Show Directories")
    show_h_f_a = models.BooleanField(default=True, verbose_name="Show Hidden Files")
    show_h_d_a = models.BooleanField(default=True, verbose_name="Show Hidden Directories")
    can_down_a = models.BooleanField(default=True, verbose_name="Can Download")
    can_up_a = models.BooleanField(default=True, verbose_name="Can Upload")
    can_del_a = models.BooleanField(default=True, verbose_name="Can Delete")
    can_crt_a = models.BooleanField(default=True, verbose_name="Can Create")
    can_run_a = models.BooleanField(default=True, verbose_name="Can Run Command")

    def __str__(self):
        return self.path

    def clean(self):
        try:
            listdir(self.path)
            if self.path[-1] != '/':
                self.path += '/'
        except FileNotFoundError:
            raise ValidationError("'{}' is not a real path on server.".format(self.path or 'Empty'))

        except NotADirectoryError:
            raise ValidationError("'{}' is not a directory.".format(self.path or 'Empty'))



from django import forms


class UploadFileForm(forms.Form):
    file = forms.FileField()


class MakeDirForm(forms.Form):
    name = forms.CharField(max_length=300)


class MakeFileForm(forms.Form):
    name = forms.CharField(max_length=300)
    content = forms.CharField(widget=forms.Textarea, required=False)



from django.http import HttpResponse
from os.path import isfile, getsize
from os import listdir, mkdir, system, stat
from wsgiref.util import FileWrapper
import tempfile
import shutil


def find_deepest_location(locations, address):
    location = None
    for lo in locations:
        if str(address).__contains__(str(lo.path)):
            if location:
                if len(str(location.path)) < len(str(lo.path)):
                    location = lo
            else:
                location = lo
    return location


def handle_delete(address, data_type):
    try:
        system("rm -rf {}".format(address))
    except:
        pass


def handle_make_file(address, content):
    try:
        with open(address, 'w+') as file:
            file.write(content)
    except:
        pass


def handle_make_dir(address):
    try:
        mkdir(address)
    except:
        pass


def handle_uploaded_file(f, address):
    with open(address, 'wb+') as destination:
        for chunk in f.chunks():
            destination.write(chunk)


def send_file(address):
    filename = address
    wrapper = FileWrapper(open(filename, 'rb'))
    response = HttpResponse(wrapper, content_type='application/force-download')
    response['Content-Disposition'] = 'attachment; filename={}'.format(address.split('/')[-1].replace(' ', '-'))
    response['Content-Length'] = getsize(filename)
    return response


def send_zipfile(address, data_type):
    formats = {
        'tar.gz': 'gztar',
        'tar.bz2': 'bztar',
        'zip': 'zip',
        'tar': 'tar',
    }
    tempdir = tempfile.mkdtemp()

    # archive = zipfile.ZipFile('{}/temp.zip'.format(tmpdir), 'w', zipfile.ZIP_DEFLATED)
    # for root, dirs, files in walk(address):
    #     for file in files:
    #         archive.write(join(root, file))
    # archive.close()

    shutil.make_archive('{}/temp'.format(tempdir), formats[data_type], address)

    wrapper = FileWrapper(open('{}/temp.{}'.format(tempdir, data_type), 'rb'))
    response = HttpResponse(wrapper, content_type='application/force-download')
    response['Content-Disposition'] = 'attachment; filename={}.{}'.format(address.replace(' ', '_')[1:-1], data_type)
    response['Content-Length'] = getsize('{}/temp.{}'.format(tempdir, data_type))
    shutil.rmtree(tempdir)
    return response


def get_content(path, s_files, s_dirs, h_files, h_dirs):
    path = '{}/'.format(path) if path[-1] != '/' else str(path)
    dirs = []
    files = []
    try:
        for ld in listdir(path):
            if isfile('{}{}'.format(path, ld)) and s_files:
                if ld.startswith('.'):
                    if h_files:
                        files.append({'name': ld, 'size': stat("{}{}".format(path, ld)).st_size})
                else:
                    files.append({'name': ld, 'size': stat("{}{}".format(path, ld)).st_size})
            elif s_dirs:
                if ld.startswith('.'):
                    if h_dirs:
                        dirs.append(ld)
                else:
                    dirs.append(ld)

    except FileNotFoundError:
        dirs = FileNotFoundError
        files = FileNotFoundError
        return dirs, files

    except NotADirectoryError:
        dirs = NotADirectoryError
        files = NotADirectoryError
        return dirs, files

    return sorted(dirs), files



# Generated by Django 2.0.8 on 2018-10-01 12:41

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(max_length=300)),
                ('active_p', models.BooleanField(default=True, verbose_name='Active')),
                ('show_f_p', models.BooleanField(default=True, verbose_name='Show Files')),
                ('show_d_p', models.BooleanField(default=True, verbose_name='Show Directories')),
                ('show_h_f_p', models.BooleanField(default=False, verbose_name='Show Hidden Files')),
                ('show_h_d_p', models.BooleanField(default=False, verbose_name='Show Hidden Directories')),
                ('can_down_p', models.BooleanField(default=False, verbose_name='Can Download')),
                ('can_up_p', models.BooleanField(default=False, verbose_name='Can Upload')),
                ('can_del_p', models.BooleanField(default=False, verbose_name='Can Delete')),
                ('can_crt_p', models.BooleanField(default=False, verbose_name='Can Create')),
                ('can_run_p', models.BooleanField(default=False, verbose_name='Can Run Command')),
                ('active_u', models.BooleanField(default=True, verbose_name='Active')),
                ('show_f_u', models.BooleanField(default=True, verbose_name='Show Files')),
                ('show_d_u', models.BooleanField(default=True, verbose_name='Show Directories')),
                ('show_h_f_u', models.BooleanField(default=False, verbose_name='Show Hidden Files')),
                ('show_h_d_u', models.BooleanField(default=False, verbose_name='Show Hidden Directories')),
                ('can_down_u', models.BooleanField(default=True, verbose_name='Can Download')),
                ('can_up_u', models.BooleanField(default=True, verbose_name='Can Upload')),
                ('can_del_u', models.BooleanField(default=False, verbose_name='Can Delete')),
                ('can_crt_u', models.BooleanField(default=True, verbose_name='Can Create')),
                ('can_run_u', models.BooleanField(default=False, verbose_name='Can Run Command')),
                ('active_a', models.BooleanField(default=True, verbose_name='Active')),
                ('show_f_a', models.BooleanField(default=True, verbose_name='Show Files')),
                ('show_d_a', models.BooleanField(default=True, verbose_name='Show Directories')),
                ('show_h_f_a', models.BooleanField(default=True, verbose_name='Show Hidden Files')),
                ('show_h_d_a', models.BooleanField(default=True, verbose_name='Show Hidden Directories')),
                ('can_down_a', models.BooleanField(default=True, verbose_name='Can Download')),
                ('can_up_a', models.BooleanField(default=True, verbose_name='Can Upload')),
                ('can_del_a', models.BooleanField(default=True, verbose_name='Can Delete')),
                ('can_crt_a', models.BooleanField(default=True, verbose_name='Can Create')),
                ('can_run_a', models.BooleanField(default=True, verbose_name='Can Run Command')),
            ],
        ),
    ]



from django.urls import path
from .views import shared_locations, location_detail, download, upload, make_dir, make_file, delete

urlpatterns = [
    path('', shared_locations, name='locations'),
    path('path<path:address>', location_detail, name='location_detail'),
    path('download<path:address>:<data_type>', download, name='download'),
    path('upload<path:address>', upload, name='upload'),
    path('makedir<path:address>', make_dir, name='makedir'),
    path('makefile<path:address>', make_file, name='makefile'),
    path('delete<path:address>:<data_type>', delete, name='delete')
]



from django.contrib import admin
from .models import Location


class LocationAdmin(admin.ModelAdmin):
    fieldsets = [
        ('', {
            'fields': (
                'path',
            ),
        }),

        ('Public', {
            'fields': (
                'active_p',
                ('show_f_p', 'show_d_p', 'show_h_f_p', 'show_h_d_p'),
                ('can_down_p', 'can_up_p', 'can_crt_p', 'can_del_p', 'can_run_p'),
            ),
            'classes': ('collapse',)
        }),

        ('User', {
            'fields': (
                'active_u',
                ('show_f_u', 'show_d_u', 'show_h_f_u', 'show_h_d_u'),
                ('can_down_u', 'can_up_u', 'can_crt_u', 'can_del_u', 'can_run_u'),
            ),
            'classes': ('collapse',)
        }),

        ('Admin', {
            'fields': (
                'active_a',
                ('show_f_a', 'show_d_a', 'show_h_f_a', 'show_h_d_a'),
                ('can_down_a', 'can_up_a', 'can_crt_a', 'can_del_a', 'can_run_a'),
            ),
            'classes': ('collapse',)
        })
    ]


admin.site.register(Location, LocationAdmin)



from django.shortcuts import render
from django.urls import reverse
from django.http import HttpResponseRedirect
from .models import Location
from .utils import *
from .forms import UploadFileForm, MakeDirForm, MakeFileForm


def get_locations(request):
    if request.user.is_authenticated:
        if request.user.is_superuser:
            return Location.objects.filter(active_a=True)
        else:
            return Location.objects.filter(active_u=True)
    else:
        return Location.objects.filter(active_p=True)


def shared_locations(request):
    return render(request, template_name='smartshare/shared_locations.html',
                  context={'locations': get_locations(request)})


def location_detail(request, address):
    location = find_deepest_location(get_locations(request), address)
    if location:
        if request.user.is_authenticated:
            if request.user.is_superuser:
                dirs, files = get_content(address, location.show_f_a, location.show_d_a, location.show_h_f_a,
                                          location.show_h_d_a)
            else:
                dirs, files = get_content(address, location.show_f_u, location.show_d_u, location.show_h_f_u,
                                          location.show_h_d_u)
        else:
            dirs, files = get_content(address, location.show_f_p, location.show_d_p, location.show_h_f_p,
                                      location.show_h_d_p)

        if dirs == FileNotFoundError:
            return render(request, 'smartshare/404.html')

        if dirs == NotADirectoryError:
            return render(request, 'smartshare/404.html')
        return render(request, template_name='smartshare/location_detail.html',
                      context={'location': location, 'address': address, 'dirs': dirs, 'files': files})
    return render(request, 'smartshare/404.html')


def download(request, address, data_type):
    location = find_deepest_location(get_locations(request), address)
    if location:
        if request.user.is_authenticated:
            if request.user.is_superuser and location.can_down_a:
                if isfile(address):
                    return send_file(address)
                elif listdir(address):
                    return send_zipfile(address, data_type)
            elif location.can_down_u and not request.user.is_superuser:
                if isfile(address):
                    return send_file(address)
                elif listdir(address):
                    return send_zipfile(address, data_type)
        elif location.can_down_p:
            if isfile(address):
                return send_file(address)
            elif listdir(address):
                return send_zipfile(address, data_type)
    return render(request, 'smartshare/404.html')


def upload(request, address):
    try:
        listdir(address)
    except:
        return render(request, 'smartshare/404.html')
    location = find_deepest_location(get_locations(request), address)
    if location:
        if request.user.is_authenticated:
            if request.user.is_superuser and location.can_up_a:
                if request.method == 'POST':
                    form = UploadFileForm(request.POST, request.FILES)
                    if form.is_valid():
                        handle_uploaded_file(request.FILES['file'], '{}{}'.format(address, request.FILES['file'].name))
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = UploadFileForm()
                return render(request, 'smartshare/upload.html', {'form': form})

            elif location.can_up_u and not request.user.is_superuser:
                if request.method == 'POST':
                    form = UploadFileForm(request.POST, request.FILES)
                    if form.is_valid():
                        handle_uploaded_file(request.FILES['file'], '{}{}'.format(address, request.FILES['file'].name))
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = UploadFileForm()
                return render(request, 'smartshare/upload.html', {'form': form})

        elif location.can_up_p:
            if request.method == 'POST':
                form = UploadFileForm(request.POST, request.FILES)
                if form.is_valid():
                    handle_uploaded_file(request.FILES['file'], '{}{}'.format(address, request.FILES['file'].name))
                    return HttpResponseRedirect(reverse('location_detail', args=(address,)))
            else:
                form = UploadFileForm()
            return render(request, 'smartshare/upload.html', {'form': form})
    return render(request, 'smartshare/404.html')


def make_dir(request, address):
    try:
        listdir(address)
    except:
        return render(request, 'smartshare/404.html')
    location = find_deepest_location(get_locations(request), address)
    if location:
        if request.user.is_authenticated:
            if request.user.is_superuser and location.can_crt_a:
                if request.method == 'POST':
                    form = MakeDirForm(request.POST)
                    if form.is_valid():
                        handle_make_dir('{}{}'.format(address, form.data['name']))
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = MakeDirForm()
                return render(request, 'smartshare/make_dir.html', {'form': form})

            elif location.can_crt_u and not request.user.is_superuser:
                if request.method == 'POST':
                    form = MakeDirForm(request.POST)
                    if form.is_valid():
                        handle_make_dir('{}{}'.format(address, form.data['name']))
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = MakeDirForm()
                return render(request, 'smartshare/make_dir.html', {'form': form})

        elif location.can_crt_p:
            if request.method == 'POST':
                form = MakeDirForm(request.POST)
                if form.is_valid():
                    handle_make_dir('{}{}'.format(address, form.data['name']))
                    return HttpResponseRedirect(reverse('location_detail', args=(address,)))
            else:
                form = MakeDirForm()
            return render(request, 'smartshare/make_dir.html', {'form': form})
    return render(request, 'smartshare/404.html')


def make_file(request, address):
    try:
        listdir(address)
    except:
        return render(request, 'smartshare/404.html')
    location = find_deepest_location(get_locations(request), address)
    if location:
        if request.user.is_authenticated:
            if request.user.is_superuser and location.can_crt_a:
                if request.method == 'POST':
                    form = MakeFileForm(request.POST)
                    if form.is_valid():
                        handle_make_file('{}{}'.format(address, form.data['name']), form.data['content'])
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = MakeFileForm()
                return render(request, 'smartshare/make_file.html', {'form': form})

            elif location.can_crt_u and not request.user.is_superuser:
                if request.method == 'POST':
                    form = MakeFileForm(request.POST)
                    if form.is_valid():
                        handle_make_file('{}{}'.format(address, form.data['name']), form.data['content'])
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = MakeFileForm()
                return render(request, 'smartshare/make_file.html', {'form': form})

        elif location.can_crt_p:
            if request.method == 'POST':
                form = MakeFileForm(request.POST)
                if form.is_valid():
                    handle_make_file('{}{}'.format(address, form.data['name']), form.data['content'])
                    return HttpResponseRedirect(reverse('location_detail', args=(address,)))
            else:
                form = MakeFileForm()
            return render(request, 'smartshare/make_file.html', {'form': form})
    return render(request, 'smartshare/404.html')


def delete(request, address, data_type):
    location = find_deepest_location(get_locations(request), address)
    if location:
        if request.user.is_authenticated:
            if request.user.is_superuser and location.can_del_a:
                if request.method == 'POST':
                    if 'Delete' in request.POST:
                        handle_delete(address, data_type)
                    address = address[:address[0:-1].rfind('/') + 1]
                    return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                return render(request, 'smartshare/delete.html', {'address': address, 'data_type': data_type})

            elif location.can_del_u and not request.user.is_superuser:
                if request.method == 'POST':
                    form = MakeFileForm(request.POST)
                    if form.is_valid():
                        handle_delete(address, data_type)
                        return HttpResponseRedirect(reverse('location_detail', args=(address,)))
                else:
                    form = MakeFileForm()
                return render(request, 'smartshare/delete.html', {'form': form})

        elif location.can_del_p:
            if request.method == 'POST':
                form = MakeFileForm(request.POST)
                if form.is_valid():
                    handle_delete(address, data_type)
                    return HttpResponseRedirect(reverse('location_detail', args=(address,)))
            else:
                form = MakeFileForm()
            return render(request, 'smartshare/delete.html', {'form': form})
    return render(request, 'smartshare/404.html')



__title__ = 'django smart share'
__version__ = '0.8.8'
__author__ = 'Abolfazl Amiri'
__author_email__ = 'aa.smpro@gmail.com'
__license__ = 'GNU General Public License v3 (GPLv3)'
__description__ = F"""django smart share {__version__}
simple django app for sharing files over http / https.
"""



import setuptools
from smartshare import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="djangosmartshare",
    version=__version__,
    author="Abolfazl Amiri",
    author_email="aa.smpro@gmail.com",
    description="simple django app for sharing files over http / https.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aasmpro/djangosmartshare",
    packages=setuptools.find_packages(exclude=['smartshareserver']),
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.1",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)
